@@ -51,9 +51,10 @@ static void read_nm_symbols(SYMBOL_SET *ss, const char *file)
 }
 
 /* Read the symbols from vmlinux */
-void read_vmlinux(const char *vmlinux)
+void read_vmlinux(const OPTIONS *options)
 {
     static char procname[] = "read_vmlinux";
+    const char *vmlinux = options->vmlinux;
     if (!vmlinux)
 	return;
     ss_init(&ss_vmlinux, "vmlinux");
@@ -84,7 +85,9 @@ void read_object(const char *object, int i)
 /* Add a new entry to the list of objects */
 static SYMBOL_SET *add_ss_object(const char *file)
 {
+    struct stat statbuf;
     int i;
+    static char const procname[] = "add_ss_object";
     /* Avoid reading any object twice. */
     for (i = 0; i < ss_objects; ++i) {
 	if (strcmp((ss_object[i])->source, file) == 0)
@@ -97,6 +100,13 @@ static SYMBOL_SET *add_ss_object(const char *file)
     if (!(ss_object[ss_objects-1] = malloc(sizeof(*ss_object[0]))))
 	malloc_error("alloc ss_object[n]");
     ss_init(ss_object[ss_objects-1], file);
+    if (stat(file, &statbuf)) {
+	ERROR("cannot stat(%s)", file);
+	perror(prefix);
+    }
+    else {
+	ss_object[ss_objects-1]->mtime = statbuf.st_mtime;
+    }
     return(ss_object[ss_objects-1]);
 }
 
@@ -107,7 +117,7 @@ static void find_objects(const char *dir)
     char *cmd, *line = NULL;
     int size = 0, files = 0;
     static char const procname[] = "find_objects";
-    static char const options[] = " -follow -name '*.o' -print";
+    static char const options[] = " -follow '(' -path '*/build' -prune ')' -o -name '*.o' -print";
 
     cmd = malloc(strlen(path_find)+1+strlen(dir)+strlen(options)+1);
     if (!cmd)
@@ -133,44 +143,15 @@ static void find_objects(const char *dir)
 	    dir, dir);
 }
 
-/* Take the user supplied list of objects which can include directories.
- * Expand directories into any *.o files.  The results are stored in
- * ss_object, leaving the user supplied options untouched.  If a ksyms
- * entry contains the name of its object file, use that by preference.
+/* Run the object paths, extracting anything that looks like a module name.
  */
-void expand_objects(const OPTIONS *options)
+static void run_object_paths(const OPTIONS *options)
 {
     struct stat statbuf;
-    int i, do_the_lot = 0;
-    SYMBOL_SET *ss;
+    int i;
     const char *file;
-    static char const procname[] = "expand_objects";
-
-    for (i = 0; i < ss_ksyms_modules; ++i) {
-	ss = ss_ksyms_module[i];
-	if (!ss->object) {
-	    do_the_lot = 1;
-	    continue;
-	}
-	file = ss->object;
-	DEBUG(1, "using %s for %s", file, ss->source);
-	if (stat(ss->object, &statbuf)) {
-	    ERROR("cannot stat(%s) for %s", file, ss->source);
-	    perror(prefix);
-	    do_the_lot = 1;
-	    continue;
-	}
-	if (statbuf.st_mtime != ss->mtime)
-	    WARNING("object %s for module %s has changed since load",
-		file, ss->source);
-	ss->related = add_ss_object(file);
-    }
-
-    if (!do_the_lot) {
-	DEBUG(1, "%s", "all ksyms modules map to specific object files");
-	return;
-    }
-    DEBUG(1, "%s", "missing modutils assists, doing it the hard way");
+    static char const procname[] = "run_object_paths";
+    DEBUG(1, "%s", "");
 
     for (i = 0; i < options->objects; ++i) {
 	file = options->object[i];
@@ -186,8 +167,96 @@ void expand_objects(const OPTIONS *options)
     }
 }
 
+/* Take the user supplied list of objects which can include directories.
+ * Expand directories into any *.o files.  The results are stored in
+ * ss_object, leaving the user supplied options untouched.  If a ksyms
+ * entry contains the name of its object file, use that by preference.
+ * If --ignore-insmod-path is set then use the first object with the
+ * same mtime and basename.
+ */
+void expand_objects(const OPTIONS *options)
+{
+    struct stat statbuf;
+    int i, do_the_lot = 0, run_objects = 0;
+    SYMBOL_SET *ss;
+    const char *file;
+    static char const procname[] = "expand_objects";
+
+    for (i = 0; i < ss_ksyms_modules; ++i) {
+	ss = ss_ksyms_module[i];
+	if (!ss->object) {
+	    do_the_lot = 1;
+	    continue;
+	}
+	file = ss->object;
+	if (options->ignore_insmod_path) {
+	    /* Use the basename and mtime from insmod */
+	    const char *basename = strrchr(file, '/'), *p;
+	    int j;
+	    if (basename)
+		++basename;
+	    else
+		basename = file;
+	    DEBUG(1, "looking for basename %s with mtime %ld for %s",
+		basename, ss->mtime, ss->source);
+	    if (!run_objects) {
+		run_object_paths(options);
+		run_objects = 1;
+	    }
+	    for (j = 0; j < ss_objects; ++j) {
+		if (ss_object[j]->mtime == ss->mtime) {
+		    p = strrchr(ss_object[j]->source, '/');
+		    if (p)
+			++p;
+		    else
+		    	p = ss_object[j]->source;
+		    if (strcmp(p, basename) == 0) {
+			DEBUG(1, "found %s for %s", ss_object[j]->source, file);
+			free((char *)(ss->object));
+			ss->object = strdup(ss_object[j]->source);
+			if (!ss->object)
+			    malloc_error(procname);
+			file = ss->object;
+			ss->related = add_ss_object(file);
+			break;
+		    }
+		}
+	    }
+	    if (j == ss_objects) {
+		DEBUG(1, "no match on basename %s with mtime %ld", basename, ss->mtime);
+		do_the_lot = 1;
+		continue;
+	    }
+	}
+	else {
+	    /* Use the full pathname from insmod */
+	    DEBUG(1, "using %s for %s", file, ss->source);
+	    if (stat(ss->object, &statbuf)) {
+		ERROR("cannot stat(%s) for %s", file, ss->source);
+		perror(prefix);
+		do_the_lot = 1;
+		continue;
+	    }
+	    if (statbuf.st_mtime != ss->mtime)
+		WARNING("object %s for module %s has changed since load",
+		    file, ss->source);
+	    ss->related = add_ss_object(file);
+	}
+    }
+
+    if (!do_the_lot) {
+	DEBUG(1, "%s", "all ksyms modules map to specific object files");
+	return;
+    }
+    DEBUG(1, "%s", "missing or mismatched modutils assists, doing it the hard way");
+    if (!run_objects) {
+	run_object_paths(options);
+	run_objects = 1;
+    }
+}
+
 /* Map a symbol type to a section code. 0 - text, 1 - data, 2 - read only data,
- * 3 - C (cannot relocate), 4 - the rest.
+ * 3 - bss, 4 - C (cannot relocate), 5 - the rest.
  */
 static int section(char type)
 {
@@ -198,15 +267,17 @@ static int section(char type)
 	return 0;
     case 'D':
     case 'd':
-    case 'B':		/* Resolved weak reference to data */
 	return 1;
     case 'R':
     case 'r':
 	return 2;
-    case 'C':
+    case 'B':
+    case 'b':
 	return 3;
-    default:
+    case 'C':
 	return 4;
+    default:
+	return 5;
     }
 }
 
@@ -216,7 +287,7 @@ static int section(char type)
 SYMBOL_SET *adjust_object_offsets(SYMBOL_SET *ss)
 {
     int i;
-    addr_t adjust[] = {0, 0, 0, 0, 0};
+    addr_t adjust[] = {0, 0, 0, 0, 0, 0};
     char type;
     SYMBOL *sk, *so;
     SYMBOL_SET *ssc;
@@ -235,6 +306,8 @@ SYMBOL_SET *adjust_object_offsets(SYMBOL_SET *ss)
 	        type = 'R';
 	    else if (strstr(sk->name, "_S.data_"))
 	        type = 'D';
+	    else if (strstr(sk->name, "_S.bss_"))
+	        type = 'B';
 	    else
 		type = ' ';
 	    /* Assumption: all sections start at relative offset 0 */
@@ -260,6 +333,6 @@ SYMBOL_SET *adjust_object_offsets(SYMBOL_SET *ss)
 	    add_symbol_n(ssc, sk->address, sk->type, 1, sk->name);
     }
     ss->related = ssc;  /* map using adjusted copy */
-    DEBUG(2, "%s %llx %llx %llx", ss->source, adjust[0], adjust[1], adjust[2]);
+    DEBUG(2, "%s text %llx rodata %llx data %llx bss %llx", ss->source, adjust[0], adjust[1], adjust[2], adjust[3]);
     return(ssc);
 }
